//libuffer.cpp - linear-interpolating buffer

#include "libuffer.hpp"
#include "RTFunc.hpp"

libuffer::libuffer(unsigned long bufsize): rwbuffer(bufsize)
{
}

float libuffer::calcInterpValue(float val1, float val2, float index)
{
	float outval, index_fract;

	switch(dir)
	{
		case dir_Forward:
			index_fract = index - (float)(float2int(index));
			outval = val1 + ((val2 - val1)*index_fract);
			break;
		case dir_Backward:
			index_fract = (float)(float2int(index+1)) - index;
			outval = val1 + ((val2 - val1)*index_fract);
			break;
	}
	return outval;
}

float libuffer::readbuffer_li(float increment)
{
	unsigned long x1, x2;
	float temp;
	
	checkfloat(&outindex, true);

	x1 = float2int(outindex);
	x2 = x1 + 1;
	if(x2 >= size)
		x2 = 0;
	temp = calcInterpValue(buf[x1], buf[x2], outindex);

	outindex += increment;

	return temp;
}

float libuffer::readbuffer_li(float increment, direction fb)
{
	unsigned long x1, x2;
	float temp;

	checkfloat(&outindex, true);

	switch(fb)
	{
		case dir_Forward:
			x1 = float2int(outindex);
			x2 = x1 + 1;
			if(x2 >= size)
				x2 = 0;
			temp = calcInterpValue(buf[x1], buf[x2], outindex);
			outindex += increment;
			break;
		case dir_Backward:
			x1 = float2int(outindex);
			x2 = x1 - 1;
			if(x2 < 0)
				x2 = (size-1);
			temp = calcInterpValue(buf[x1], buf[x2], outindex);
			outindex -= increment;
			break;
	}

	return temp;
}

float libuffer::readbuffer_li(float increment, float fb)
{
	unsigned long x1, x2;
	float temp;

	checkfloat(&outindex, true);

	if(fb > 0.5f)
	{
		dir = dir_Forward;
		x1 = float2int(outindex);
		x2 = x1 + 1;
		if(x2 >= size)
			x2 = 0;
		temp = calcInterpValue(buf[x1], buf[x2], outindex);
		outindex += increment;
	}
	else
	{
		dir = dir_Backward;
		x1 = float2int(outindex+1);
		x2 = x1 - 1;
		if(x2 < 0)
			x2 = (size-1);
		temp = calcInterpValue(buf[x1], buf[x2], outindex);
		outindex -= increment;
	}

	return temp;
}

float libuffer::readbuffer_li(float increment, float fb, float looping)
{
	unsigned long x1, x2;
	float temp;

	if(looping < 0.5f)
		loop = false;
	else
	{
		loop = true;
		current = true;
	}

	checkfloat(&outindex, true);

	if(fb > 0.5f)
	{
		dir = dir_Forward;
		x1 = float2int(outindex);
		x2 = x1 + 1;
		if(x2 >= size)
			x2 = 0;
		temp = calcInterpValue(buf[x1], buf[x2], outindex);
		if((loop == true)||((loop == false)&&(current == true)))
			outindex += increment;
	}
	else
	{
		dir = dir_Backward;
		x1 = float2int(outindex+1);
		x2 = x1 - 1;
		if(x2 < 0)
			x2 = (size-1);
		temp = calcInterpValue(buf[x1], buf[x2], outindex);
		if((loop == true)||((loop == false)&&(current == true)))
			outindex -= increment;
	}

	return temp;
}