//lookupsine.cpp - 2048-point lookup table sine generator, w/linear interpolation

#include <math.h>
#include "lookupsine.hpp"

lookupsine::lookupsine(float sr, float freq):libuffer(2048)
{
	int i;
	float temp;

	samplerate = sr;
	frequency = freq;

	for(i=0;i<2048;i++)
	{
		temp = (float)i/2048.0f;
		temp *= TWO_PI;
		temp = (float)sin(temp);
		writebuffer(temp);
	}

	calcIncrement();
}

void lookupsine::fakeconstructor(float sr, float freq)
{
	int i;
	float temp;

	size = 2048;

	dir = dir_Forward;
	inindex = 0.0f;
	outindex = 0.0f;
	loop = false;
	current = false;

	buf = new float[size];

	samplerate = sr;
	frequency = freq;

	for(i=0;i<2048;i++)
	{
		temp = (float)i/2048.0f;
		temp *= TWO_PI;
		temp = (float)sin(temp);
		writebuffer(temp);
	}

	calcIncrement();
}

void lookupsine::calcIncrement()
{
	float temp;

	temp = samplerate/frequency;

	increment = 2048.0f/temp;
}


float lookupsine::getsample(float freq)
{
	if(frequency != freq)
	{
		frequency = freq;

		calcIncrement();
	}

	return readbuffer_li(increment);
}

void lookupsine::resetPhase()
{
	resetoutindex();
}