//rwbuffer.cpp - simple buffer object, w/multiple read & write methods (stores floats)

#include "rwbuffer.hpp"
#include <memory.h>

rwbuffer::rwbuffer(unsigned long bufsize)
{
  unsigned long i;

  size = bufsize;

  dir = dir_Forward;
  inindex = 0.0f;
  outindex = 0.0f;
  loop = false;
  current = false;

  buf = new float[size];

  for(i=0;i<size;i++)   //initialise buffer to zero
	  buf[i] = 0.0f;
}

rwbuffer::~rwbuffer()
{
	delete [] buf;
}

void rwbuffer::checkfloat(float *index, bool io)
{
	float temp1, temp2;

	temp1 = *index;

	switch(io)
	{
		case true: //true = out
			switch(dir)
			{
				case dir_Forward:
					if(temp1 < 0.0f)
						temp1 *= -1.0f;
					if (temp1 >= size)
					{
						temp2 = (float)((float)temp1-float2int(temp1));
						temp1 = (float)(float2int(temp1) % size);
						temp1 += temp2;
						if(loop == false)
							current = false;
					}
					break;
				case dir_Backward:
					if(temp1 < 0.0f)
					{
						temp1 += size;
						if(loop == false)
							current = false;
					}
					if (temp1 >= size)
					{
						temp2 = (float)((float)temp1-float2int(temp1));
						temp1 = (float)(float2int(temp1) % size);
						temp1 += temp2;

						temp1 = ((float)size-1.0f)-temp1;
					}
					break;
			}
			break;
		case false: //false = in
			if(temp1 < 0.0f)
				temp1 *= -1.0f;
			if (temp1 >= size)
			{
				temp2 = (float)((float)temp1-float2int(temp1));
				temp1 = (float)(float2int(temp1) % size);
				temp1 += temp2;
			}
			break;
	}

	*index = temp1;
}

int rwbuffer::checkindex(float *index, bool io)
{
	int tindex;
	float temp;

	checkfloat(index, io);

	temp = *index;

	tindex = float2int(temp);

	return tindex;
}

void rwbuffer::writebuffer(float value)
{
	int temp;

	temp = checkindex(&inindex, false);

	buf[temp] = value;

	inindex++;
}

void rwbuffer::writebuffer(float value, float increment)//dunno why anyone'd want
{														//to use this one...
	int temp;

	temp = checkindex(&inindex, false);

	buf[temp] = value;

	inindex += increment;
}

void rwbuffer::writebuffer(float *buffer)
{
	unsigned long i;

	for(i=0;i<size;i++)
		buf[i] = buffer[i];

	delete [] buffer;
	buffer = 0;
}

float rwbuffer::readbuffer()
{
	int temp;

	temp = checkindex(&outindex, true);

	outindex ++;

	return buf[temp];
}

float rwbuffer::readbuffer(float increment)
{
	int temp;

	temp = checkindex(&outindex, true);

	outindex += increment;

	return buf[temp];
}

float rwbuffer::readbuffer(direction fb)
{
	int temp;

	temp = checkindex(&outindex, true);

	switch(fb)
	{
		case dir_Forward:
			outindex++;
			break;
		case dir_Backward:
			outindex--;
			break;
	}

	return buf[temp];
}

float rwbuffer::readbuffer(float increment, direction fb)
{
	int temp;

	temp = checkindex(&outindex, true);

	switch(fb)
	{
		case dir_Forward:
			outindex += increment;
			break;
		case dir_Backward:
			outindex -= increment;
			break;
	}

	return buf[temp];
}

float rwbuffer::readbuffer(float increment, float fb)//takes a float value as a direction
{													 //- this is easier to use in a VST plugin
	int temp;

	temp = checkindex(&outindex, true);

	if(fb > 0.5f)
	{
		dir = dir_Forward;
		outindex += increment;
	}
	else
	{
		dir = dir_Backward;
		outindex -= increment;
	}

	return buf[temp];
}

float rwbuffer::readbuffer(float increment, float fb, float looping)
{
	int temp;

	if(looping < 0.5f)
		loop = false;
	else
	{
		loop = true;
		current = true;
	}

	temp = checkindex(&outindex, true);

	if(fb > 0.5f)
	{
		dir = dir_Forward;
		if((loop == true)||((loop == false)&&(current == true)))
			outindex += increment;
	}
	else
	{
		dir = dir_Backward;
		if((loop == true)||((loop == false)&&(current == true)))
			outindex -= increment;
	}

	return buf[temp];
}

void rwbuffer::resetoutindex()
{
	if(dir == dir_Forward)
		outindex = 0.0f;
	else
		outindex = (float)size;
	current = true;
}

void rwbuffer::resetout2inindex()
{
	outindex = inindex - 1.0f;
}

void rwbuffer::clearBuff()
{
	unsigned long it;

	for(it=0;it<size;it++)
		buf[it] = 0.0f;
}