//sixparamenvelope.cpp - Niall's daft 6-parameter envelope (AHDAHR)
//   _  _
//  / \/ \
// /      \

#include "sixparamenvelope.hpp"

sixparamenvelope::sixparamenvelope(float sr)
{
	samplerate = sr;

	timeIncrement = 1.0f/sr;
	currTime = 0.0f;
	currSection = 0;
	envelope = 0.0f;

	time[0] = 0.5f;
	time[1] = 0.5f;
	time[2] = 0.5f;
	time[3] = 0.5f;
	time[4] = 0.5f;
	time[5] = 0.5f;

	mLev = 0.5f;

	mult1 = calcMult(time[0], 0.0f, 1.0f);
	mult3 = calcMult(time[2], 1.0f, mLev);
	mult4 = calcMult(time[3], mLev, 1.0f);
	mult6 = calcMult(time[5], 1.0f, 0.0f);
}

sixparamenvelope::~sixparamenvelope()
{

}

float sixparamenvelope::getsample(float tim1,		//A
								  float tim2,		//H
								  float tim3,		//D,A
								  float tim4,		//H
								  float tim5,		//R
								  float midLev,	// \/ Level
								  bool OnOff,		//is the envelope on?
								  bool Looping)	//is the envelope looping?
{
	if(OnOff)
	{
		mLev = midLev;

		currTime += timeIncrement;
		if(currTime > time[currSection])
		{
			currSection++;
			if((currSection > 5)&&Looping)
				currSection = 0;
			else if(!Looping)
				return 0.0f;

			currTime = 0.0f;
			if(currSection == 0)
				envelope = 0.0f;
		}

		switch(currSection)
		{
			case 0:
				time[0] = (tim1+0.0001f) * 2.0f;
				mult1 = calcMult(time[0], 0.0f, 1.0f);
				envelope += mult1;
				break;
			case 1:
				time[1] = (tim2+0.0001f) * 2.0f;
				envelope = 1.0f;
				break;
			case 2:
				time[2] = (tim3+0.0001f);
				mult3 = calcMult(time[2], 1.0f, mLev);
				envelope -= mult3;
				break;
			case 3:
				time[3] = (tim3+0.0001f);
				mult4 = calcMult(time[3], mLev, 1.0f);
				envelope += mult4;
				break;
			case 4:
				time[4] = (tim4+0.0001f) * 2.0f;
				envelope = 1.0f;
				break;
			case 5:
				time[5] = (tim5+0.0001f) * 2.0f;
				mult6 = calcMult(time[5], 1.0f, 0.0f);
				envelope -= mult6;
				break;
		}

		if(envelope > 1.0f)
		{
			/*currSection++;
			currTime = 0.0f;*/
			envelope = 1.0f;
		}
		else if(envelope < 0.0f)
		{
			/*currSection++;
			currTime = 0.0f;*/
			envelope = 0.0f;
		}

		return envelope;
	}
	else
		return 0.0f;
}

float sixparamenvelope::calcMult(float time, float start, float end)
{
	float retval;
	float noIntervals, ydist;

	noIntervals = time*samplerate;

	if(start > end)
		ydist = start - end;
	else
		ydist = end - start;

	retval = ydist/noIntervals;

	return retval;
}